"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var BaseAction_1 = require("./BaseAction");
var MetaReducer_1 = require("../reducer/MetaReducer");
var meta_1 = require("../utils/meta");
var dev_1 = require("../../libs/dev");
var dev_2 = require("../utils/dev");
var BaseRequestAction = /** @class */ (function (_super) {
    tslib_1.__extends(BaseRequestAction, _super);
    function BaseRequestAction(request, instanceName, runAction) {
        var _this = _super.call(this, instanceName) || this;
        _this.prepareType = _this.typePrefix + " prepare";
        _this.failType = _this.typePrefix + " fail";
        if (!dev_1.isDebug() || !dev_2.isProxyEnable()) {
            _this.registerMetas();
        }
        // @ts-ignore
        return _this.proxy(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var action = request.apply(void 0, tslib_1.__spread(args))
                .collect({
                prepare: _this.prepareType,
                success: _this.successType,
                fail: _this.failType,
            });
            return runAction(action);
        }, [
            'onSuccess', 'onPrepare', 'onFail',
            'getPrepareType', 'getFailType',
            'useMeta', 'useMetas', 'useLoading', 'useLoadings',
        ], [
            'meta', 'metas', 'loading', 'loadings',
        ]);
    }
    BaseRequestAction.prototype.onSuccess = function (effect) {
        return {
            when: this.successType,
            effect: effect,
        };
    };
    BaseRequestAction.prototype.onPrepare = function (effect) {
        return {
            when: this.prepareType,
            effect: effect,
        };
    };
    BaseRequestAction.prototype.onFail = function (effect) {
        return {
            when: this.failType,
            effect: effect,
        };
    };
    BaseRequestAction.prototype.getPrepareType = function () {
        return this.prepareType;
    };
    BaseRequestAction.prototype.getFailType = function () {
        return this.failType;
    };
    BaseRequestAction.prototype.useMeta = function (key) {
        var _this = this;
        return this.switchReduxSelector()(function (state) {
            var customMeta = state[MetaReducer_1.MetaReducer.reducerName][_this.typePrefix];
            if (customMeta === undefined) {
                customMeta = meta_1.DEFAULT_META;
            }
            return key ? customMeta[key] : customMeta;
        });
    };
    BaseRequestAction.prototype.useMetas = function (value, metaKey) {
        var _this = this;
        if (value === undefined) {
            metaKey = undefined;
        }
        return this.switchReduxSelector()(function (state) {
            var customMetas = state[MetaReducer_1.MetaReducer.reducerName][_this.typePrefix];
            if (customMetas === undefined) {
                customMetas = meta_1.DEFAULT_METAS;
            }
            var customMeta = value === undefined ? customMetas : customMetas[value] || meta_1.DEFAULT_META;
            return metaKey
                ? customMeta[metaKey]
                : customMeta;
        });
    };
    BaseRequestAction.prototype.useLoading = function () {
        return this.useMeta('loading');
    };
    BaseRequestAction.prototype.useLoadings = function (value) {
        return value === undefined
            ? this.getLoadingHandle(this.useMetas())
            : this.useMetas(value, 'loading');
    };
    Object.defineProperty(BaseRequestAction.prototype, "meta", {
        get: function () {
            return MetaReducer_1.MetaReducer.getData(this.typePrefix) || meta_1.DEFAULT_META;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseRequestAction.prototype, "metas", {
        get: function () {
            return MetaReducer_1.MetaReducer.getData(this.typePrefix) || meta_1.DEFAULT_METAS;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseRequestAction.prototype, "loading", {
        get: function () {
            return this.meta.loading;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseRequestAction.prototype, "loadings", {
        get: function () {
            return this.getLoadingHandle(this.metas);
        },
        enumerable: true,
        configurable: true
    });
    BaseRequestAction.prototype.getLoadingHandle = function (metas) {
        if (!this.loadingsCache || this.loadingsCache[0] !== metas) {
            this.loadingsCache = [metas, {
                    pick: function (payload) {
                        return metas.pick(payload).loading;
                    },
                }];
        }
        return this.loadingsCache[1];
    };
    BaseRequestAction.prototype.onTypePrefixChanged = function () {
        _super.prototype.onTypePrefixChanged.call(this);
        this.prepareType = this.typePrefix + " prepare";
        this.failType = this.typePrefix + " fail";
        if (dev_1.isDebug() && dev_2.isProxyEnable()) {
            this.registerMetas();
        }
    };
    BaseRequestAction.prototype.registerMetas = function () {
        var types = {
            prepare: this.prepareType,
            success: this.successType,
            fail: this.failType,
        };
        MetaReducer_1.MetaReducer.addCase(this.typePrefix, types);
    };
    return BaseRequestAction;
}(BaseAction_1.BaseAction));
exports.BaseRequestAction = BaseRequestAction;
