"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var method_1 = require("../utils/method");
var OrphanHttpServiceHandle_1 = require("./OrphanHttpServiceHandle");
var createReduxStore_1 = require("../utils/createReduxStore");
var RequestAction_1 = require("../../libs/RequestAction");
var instanceName_1 = require("../utils/instanceName");
var dev_1 = require("../../libs/dev");
var dev_2 = require("../utils/dev");
var BaseHttpService = /** @class */ (function () {
    function BaseHttpService(config) {
        this.config = config;
    }
    BaseHttpService.prototype.action = function (fn) {
        var instanceName = instanceName_1.getInstanceName();
        if (!dev_1.isDebug() || !dev_2.isProxyEnable()) {
            instanceName += '_' + instanceName_1.increaseActionCounter();
        }
        return new RequestAction_1.RequestAction(fn, instanceName, this.runAction.bind(this));
    };
    BaseHttpService.prototype.getAsync = function (config) {
        return new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, this)
            .setMethod(method_1.METHOD.get)
            .runAction();
    };
    BaseHttpService.prototype.postAsync = function (config) {
        return new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, this)
            .setMethod(method_1.METHOD.post)
            .runAction();
    };
    BaseHttpService.prototype.putAsync = function (config) {
        return new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, this)
            .setMethod(method_1.METHOD.put)
            .runAction();
    };
    BaseHttpService.prototype.deleteAsync = function (config) {
        return new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, this)
            .setMethod(method_1.METHOD.delete)
            .runAction();
    };
    BaseHttpService.prototype.next = function (action) {
        if (action.type) {
            createReduxStore_1.getStore().dispatch(action);
        }
    };
    BaseHttpService.prototype.triggerShowError = function (errorResponse, hideError) {
        if (!errorResponse.message) {
            return;
        }
        var showError;
        if (typeof hideError === 'boolean') {
            showError = !hideError;
        }
        else {
            showError = !hideError(errorResponse);
        }
        if (showError) {
            this.config.onShowError(errorResponse.message, errorResponse);
        }
    };
    return BaseHttpService;
}());
exports.BaseHttpService = BaseHttpService;
