"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var redux_1 = require("redux");
var StoreNotFoundError_1 = require("../exceptions/StoreNotFoundError");
var dev_1 = require("../../libs/dev");
var hasEffectsReducers = [];
var autoReducers = {};
var usersReducers = {};
var store;
var listeners = [];
var onCombineReducers;
var combine = function () {
    if (dev_1.isDebug()) {
        Object.keys(usersReducers).forEach(function (key) {
            if (autoReducers[key] && hasEffectsReducers.indexOf(key) === -1) {
                // Indeed, it's reducer name but not model name
                console.warn("Model '" + key + "' has been registered automatically, do not register it again.");
            }
        });
    }
    var combined = redux_1.combineReducers(tslib_1.__assign(tslib_1.__assign({}, autoReducers), usersReducers));
    if (onCombineReducers) {
        return onCombineReducers(combined);
    }
    return combined;
};
exports.watchEffectsReducer = function (reducerName, className) {
    hasEffectsReducers.push(reducerName);
    setTimeout(function () {
        if (!usersReducers[reducerName]) {
            console.error("Model '" + className + "' has override protected method 'effects()', consider register its instance manually:\n\n\nExample:\n\nconst store = createReduxStore({ \n  reducers: {\n    ...aModel.register(),\n    ...bModel.register(),\n    ...cModel.register(),\n  }\n});\n\n\n");
        }
    });
};
exports.appendReducers = function (reducers) {
    Object.assign(autoReducers, reducers);
    if (store) {
        store.replaceReducer(combine());
    }
};
function createReduxStore(config) {
    usersReducers = config.reducers || {};
    onCombineReducers = config.onCombineReducers;
    if (store) {
        store.replaceReducer(combine());
    }
    else {
        store = redux_1.createStore(combine(), config.preloadedState, config.enhancer);
        listeners.forEach(function (listener) { return listener(store); });
        listeners = [];
    }
    return store;
}
exports.createReduxStore = createReduxStore;
exports.getStore = function () {
    if (!store) {
        throw new StoreNotFoundError_1.StoreNotFoundError();
    }
    return store;
};
exports.onStoreCreated = function (fn) {
    if (store) {
        fn(store);
    }
    else {
        listeners.push(fn);
    }
};
